@@ -1,8 +1,8 @@
-/* cups-pdf.c -- CUPS Backend 
+/* cups-pdf.c -- CUPS Backend (version 2.5.0, 2009-01-26)
    08.02.2003, Volker C. Behr
-   Exp. Physik V, Uni Wuerzburg 
-   vrbehr@cip.physik.uni-wuerzburg.de
-   http://cip.physik.uni-wuerzburg.de/~vrbehr/cups-pdf
+   Experimentelle Physik V, Universitaet Wuerzburg 
+   behr@physik.uni-wuerzburg.de
+   http://www.cups-pdf.de
 
    This code may be freely distributed as long as this header 
    is preserved. 
@@ -12,7 +12,7 @@
 
    ---------------------------------------------------------------------------
 
-   Copyright (C) 2003-2008  Volker C. Behr
+   Copyright (C) 2003-2009  Volker C. Behr
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -37,101 +37,8 @@
    I would appreciate your feedback via email.
 
   
-   HISTORY: 
-   2008-06-22 : 2.4.8      - fixed too small allocation of postprocessing string,
-                             corrected typo in the config file documentation,
-                             update for the SELinux .te-file in contrib/
-   2008-03-24 : 2.4.7      - corrected the exit codes to match CUPS' specifications,
-                             unmodified username passed to PostProcessing as 3rd arg,
-                             made PPD file auto-selectable via IEEE-1284 device id
-   2007-05-04 : 2.4.6      - optimized default options for GS to produce PDFs
-   2007-02-27 : 2.4.5      - enclosing parentheses () are removed from title,
-                             set a more comprehensive identification string
-   2007-02-02 : 2.4.4      - DOS-style path separators '\' are now recognized
-   2006-11-22 : 2.4.3      - replaced 'setenv' by 'putenv' to increase compatibility
-   2006-09-30 : 2.4.2      - several code opimizations,
-                             PostScript PPD now uncompressed and rev4 only
-   2006-06-13 : 2.4.1      - added a PDF-to-mail postprocessing script to contrib/,
-                             added a new revision of the PostscriptColor PPD file
-   2006-05-28 : 2.4.0      - created the contrib/ directory with additional software
-   2006-05-17 : 2.3.1	   - fixed possible read access after end of string,
-                             fixed recognition of international characters 
-   2006-05-14 : 2.3.0	   - experimental option to decode hex strings in title (NK)
-   2006-04-14 : 2.2.0      - new option to remove prefixes from usernames
-   2006-04-06 : 2.1.1      - minor bugfix to read umasks properly
-   2006-03-25 : 2.1.0	   - new option to select where to look for title,
-                             some code changes and optimizations
-   2006-02-26 : 2.0.5      - several code optimizations
-   2006-01-29 : 2.0.4      - beginning of (E)PS-code now determined by "%!",
-                             optimized system call to GS in cups-pdf.conf,
-              		     some minor code changes,
-			     debug information shows version number,
-      			     updated PostScript PPD file supporting ledger paper
-   2006-01-07 : 2.0.3	   - improved checking for root privileges (incl. EUID)
-   2006-01-04 : 2.0.2	   - fixed possible testing of an uninitialized variable
-   2006-01-03 : 2.0.1	   - fixed hanging jobs when logging is disabled
-   2005-12-23 : 2.0.0      - creation of non-existent parent directories,
-                             improved configurability of output directories,
-			     option CPHOMESUB is now obsolete,
-                             PPD file now matches Adobe specifications
-   2005-10-23 : 2.0beta2   - added support for post-processing script,
-                             fixed a bug when parsing cups-pdf.conf
-   2005-09-18 : 2.0beta1   - added support for runtime configuration,
-                             introduced cups-pdf.conf,
-                             made AnonDirName an abolute path
-   2005-09-08 : 1.7.4      - added some options to cups-pdf.h for MacOSX
-   2005-07-23 : 1.7.3      - fixed GhostScript failure with error 11,
-                             fixed segfault when called without root privileges
-   2005-07-07 : 1.7.2	   - adjustment of file mode moved to non-privileged child
-   2005-06-16 : 1.7.1	   - PDFs are protected during creation by file mode 000
-   2005-03-07 : 1.7.0a
-   2005-03-01 : 1.7.0      - call of GhostScript from a non-privileged child process,
-                             no more additional programs in the call of GhostScript,
-                             several code optimizations
-   2005-02-09 : 1.6.6      - updated PostScript PPD file supporting more DIN formats
-   2005-01-04 : 1.6.5      - cups-pdf is running with gid of group defined in conf.grp
-   2004-10-11 : 1.6.4      - improved configurability for different versions of 'su'
-   2004-10-07 : 1.6.3      - corrected erroneous call of GhostScript
-   2004-10-05 : 1.6.2      - added necessary quotes in the call of GhostScript
-   2004-10-03 : 1.6.1      - allow users w/o valid shell in /etc/passwd,
-                             several code optimizations
-   2004-09-08 : 1.6.0      - new option to check against user names in lower case,
-                             new option to label all created PDFs with a job-id
-   2004-08-10 : 1.5.2      - fixed insecure creation of spoolfile,
-     			     several smaller code optimizations
-   2004-08-09 : 1.5.1	   - fixed possible access to element outside allocated area,
-                             fixed possible conflict with overlapping memory areas
-   2004-08-03 : 1.5.0      - call to GS without root privileges for enhanced security
-   2004-08-01 : 1.4.3      - added -dSAFER to the call to GS to improve security
-   2004-02-25 : 1.4.2a
-   2004-02-24 : 1.4.2      - improved extraction of filenames,
-                             handling of embedded (e)ps code (RVT)
-   2004-02-09 : 1.4.1	   - logfile is created with mode 644 if it does not exist
-   2004-01-27 : 1.4.0      - new option to set permissions for PDF output,
-                             alternative options for improved image conversion
-                             (moved conf.gscall to cups-pdf.h)
-   2004-01-06 : 1.3.2	   - exit status of ghostscript gets logged in debug mode
-   2004-01-02 : 1.3.1a
-   2003-11-29 : 1.3.1	   - improved handling of domain prefix
-   2003-11-02 : 1.3	   - new option to cut file name extensions
-   2003-10-25 : 1.2        - untitled documents are labelled with job-id,
-			     new option to add a domain prefix to user names,
-                             removed a minor bug during character replacement
-   2003-08-02 : 1.1	   - files not viewable until creation is finished,
-   			     new qualifier "$HOME" for output directory,
-			     "$HOME"-related new define in the header file
-   2003-07-17 : 1.0.1	   - improved user name handling for multi-word names,
-                             code now distributed under the GPL
-   2003-06-29 : 1.0	   - enhanced debug output,
-   			     replacement of special characters in title,
-			     new option in the header file
-   2003-05-26 : 1.0pre3    - several minor bugfixes and improvements 
-   2003-05-21 : 1.0pre2    - optimized memory management
-   2003-05-16 : 1.0pre1    - introduced cups-pdf.h, 
-			     code now entirely written in C,
- 			     cups-pdfgen is obsolete
-   2003-02-14 : 0.2    - script re-located to /usr/lib/cups/filter/cups-pdfgen
-   2003-02-12 : 0.1    - first beta release			                */
+   HISTORY: see ChangeLog in the parent directory of the source archive
+*/
 
 #include <time.h>
 #include <errno.h>
@@ -506,6 +413,10 @@ static int preparetitle(char *title) {
   }
   else
     replace_string(title);
+  if (strlen(title)>conf.truncate) {
+    title[conf.truncate]='\0';
+    log_event(CPDEBUG, "truncating title", title);
+  }
   return strcmp(title, "");
 }
 
@@ -745,6 +656,8 @@ int main(int argc, char *argv[]) {
   outfile=calloc(size, sizeof(char));
   if (outfile == NULL) {
     (void) fputs("CUPS-PDF: failed to allocate memory\n", stderr);
+    if (unlink(spoolfile))
+      log_event(CPERROR, "failed to unlink spoolfile during clean-up", spoolfile);
     free(dirname);
     free(spoolfile);
     if (logfp!=NULL)
@@ -758,6 +671,8 @@ int main(int argc, char *argv[]) {
   gscall=calloc(size, sizeof(char));
   if (gscall == NULL) {
     (void) fputs("CUPS-PDF: failed to allocate memory\n", stderr);
+    if (unlink(spoolfile))
+      log_event(CPERROR, "failed to unlink spoolfile during clean-up", spoolfile);
     free(dirname);
     free(spoolfile);
     free(outfile);
@@ -773,6 +688,8 @@ int main(int argc, char *argv[]) {
 
   if (putenv(conf.gstmp)) {
     log_event(CPERROR, "insufficient space in environment to set TMPDIR", conf.gstmp);
+    if (unlink(spoolfile))
+      log_event(CPERROR, "failed to unlink spoolfile during clean-up", spoolfile);
     free(dirname);
     free(spoolfile);
     free(outfile);
@@ -803,11 +720,10 @@ int main(int argc, char *argv[]) {
     size=system(gscall);
     snprintf(title,BUFSIZE,"%d",size);
     log_event(CPDEBUG, "ghostscript has finished", title);
-    if (chmod(outfile, mode)) {
+    if (chmod(outfile, mode))
       log_event(CPERROR, "failed to set file mode for PDF file (non fatal)", outfile);
-      return 5;
-    }
-    log_event(CPDEBUG, "file mode set for user output", outfile);
+    else 
+      log_event(CPDEBUG, "file mode set for user output", outfile);
     
     if (strlen(conf.postprocessing)) {
       size=strlen(conf.postprocessing)+strlen(outfile)+strlen(passwd->pw_name)+strlen(argv[2])+4;
